'''
datalayer.py - data handling

----
Copyright (C) 2007 Zoran Isailovski
Licensed under the Apache License, Version 2.0 (see http://www.apache.org/licenses/LICENSE-2.0)
or, at your option, under the terms described in the file LICENSE.txt found in the root of the 
distribution of this package (as far as it denotes a different license then the above).
'''

######################################################################
## 
## rating storage interface
## 
######################################################################
######################################################################
######################################################################

class IRatingStorage:
    def Rate(self, pagename, itemname, username, value):
        raise NotImplementedError
    def GetRankInfo(self, pagename, itemname, username):
        "-> RankInfo"
        raise NotImplementedError

class RankInfo:
    def __init__(self, nvotes, average, mine):
        self.nvotes = nvotes
        self.average = average
        self.mine = mine

######################################################################
######################################################################
######################################################################
## 
## Implementations
## 
######################################################################
import os

######################################################################
## 
## PerSubjectSavedMiniModel
## 
######################################################################

class PerSubjectSavedMiniModel(IRatingStorage):
    def __init__(self, storageFolder, subject):
        self.filename = self.FilenameFor(storageFolder, subject)
        self.count = 0
        self.total = self.average = 0.0
        self._Load()
    @staticmethod
    def FilenameFor(storageFolder, subject):
        return os.path.join(storageFolder, '.rating-%s.state' % subject)
    def Rate(self, pagename, itemname, username, value):
        self.total += float(value)
        self.count += 1
        self.average = self.total / self.count
        self._Store()
    def GetRankInfo(self, pagename, itemname, username):
        return RankInfo(self.count, self.average, self.average)
    def _Load(self):
        if os.path.isfile(self.filename):
            s = fread(self.filename)
            self.count, self.total, self.average = eval(s)
    def _Store(self):
        data = (self.count, self.total, self.average)
        fwrite(self.filename, data)


######################################################################
## 
## Sohisticated Rating
## 
######################################################################

class PickledModel(IRatingStorage):
    def __init__(self, filename):
        self.filename = filename
        self.data = unpickleFrom(self.filename, {})
    def Rate(self, pagename, itemname, username, value):
        item = self.Rating(itemname)
        item.Rate(value, username)
        pickleTo(self.filename, self.data)
    def GetRankInfo(self, pagename, itemname, username):
        item = self.Rating(itemname)
        return RankInfo(len(item.votes), item.Rank, item.VoteOf(username))
    def Rating(self, itemname):
        item = self.data.get(itemname, None)
        if not item:
            self.data[itemname] = item = RatableItem()
        return item

class RatableItem:
    def __init__(self):
        self.votes = {}
        ##self.count = 0
        self._rank = 0.0
    def Rate(self, value, username):
        self.votes[username] = float(value)
        self._rank = None
    def Unrate(self, username):
        try:
            del self.votes[username]
        except KeyError:
            pass
        else:
            self._rank = None
    @property
    def Rank(self):
        if self._rank is None: self._rank = sum(self.votes.values()) / len(self.votes)
        return self._rank
    def VoteOf(self, username):
        return self.votes.get(username, 0.0)

######################################################################
## 
## PerSubjectPickledModel
## 
######################################################################

class PerSubjectPickledModel(PickledModel):
    def __init__(self, storageFolder, subject):
        filename = self.FilenameFor(storageFolder, subject)
        PickledModel.__init__(self, filename)
    @staticmethod
    def FilenameFor(storageFolder, subject):
        return os.path.join(storageFolder, '.rating-%s.data' % subject)

######################################################################
## 
## File IO Auxiliaries
## 
######################################################################

def fread(fname):
    f = open(fname, 'rt')
    try: return f.read()
    finally: f.close()

def fwrite(fname, content):
    f = open(fname, 'wt')
    try: return f.write(str(content))
    finally: f.close()

######################################################################
## 
## Pickle IO Auxiliaries
## 
######################################################################
try:
    import cPickle as pickle
except ImportError:
    import pickle

def unpickleFrom(fname, default=None):
    if not os.path.isfile(fname): return callable(default) and default() or default
    f = open(fname, 'rb')
    try: return pickle.load(f)
    finally: f.close()

def pickleTo(fname, obj):
    f = open(fname, 'wb')
    try: return pickle.dump(obj, f, -1)
    finally: f.close()

